/*  Compile this with:
 * 
 *      gcc -o fftw-example -g fftw-example.c -lsrfftw -lsfftw -lm
 */

#include <stdio.h>

#include <math.h>
#include <fftw.h>
#include <rfftw.h>

#include <string.h>

#define MAX_X           23
#define MAX_Y           17



char buf[1024];

#define IN dofoo(argv,"p")
#define FRQ dofoo(argv,"f")
#define KILL "kill.ppm"
#define OUT dofoo(argv,"o")

char *
dofoo (char **a, char *p)
{
  static char fn[1024];
  strcpy (fn, p);
  strcat (fn, *a);

  return fn;


}

void
fgetsnc (char *buf, int i, FILE * f)
{
  buf[0] = '#';

  while (buf[0] == '#')
    fgets (buf, i, f);
}

#define NX 1024
#define NY 1024
#define NQ ((NX/2)+1)
#define OD (NX*NY)
#define OD2 (NX*NY*2)
#define OF (NY*NQ)
#define OF2 (NY*NQ*2)

int
main (int argc, char *argv[])
{
  fftw_real *data;
  fftw_complex *fft;
  rfftwnd_plan fplan, bplan;
  FILE *kl,*ws;

//Wisdom

  ws=fopen("wisdom","w+");
  fftw_import_wisdom_from_file(ws);
  
  printf ("Planning...\n");
  fplan =
    rfftw2d_create_plan (NY, NX, FFTW_REAL_TO_COMPLEX,
                         FFTW_MEASURE | FFTW_OUT_OF_PLACE);
  printf ("Plotting...\n");

  bplan =
    rfftw2d_create_plan (NY, NX, FFTW_COMPLEX_TO_REAL,
                         FFTW_ESTIMATE | FFTW_OUT_OF_PLACE);


  data = malloc (sizeof (fftw_real) * NX * NY * 3);
  bzero (data, sizeof (fftw_real) * NX * NY * 3);

  fft = malloc (sizeof (fftw_complex) * NY * NQ * 3);
  bzero (fft, sizeof (fftw_real) * NY * NQ * 3);

  argc--;
  argv++;

  while (argc--)
    {
      printf ("Reading image %s\n", IN);
      {
        FILE *in = fopen (IN, "r");

        fgetsnc (buf, sizeof (buf), in);
        fgetsnc (buf, sizeof (buf), in);
        {
          int x, y;

          sscanf (buf, "%d %d", &x, &y);
          if ((x != NX) || (y != NY))
            {
              printf ("image has wrong size\n");
            }

        }
        fgetsnc (buf, sizeof (buf), in);

        {
          int n = NX * NY;
          fftw_real *ptr = data;

          while (n--)
            {
              *ptr = ((fftw_real) (getc (in))) / 255.0f;
              *(ptr + OD) = ((fftw_real) (getc (in))) / 255.0f;
              *(ptr + OD2) = ((fftw_real) (getc (in))) / 255.0f;
              ptr++;
            }
        }
        fclose (in);
      }

      printf ("Planning\n");


      printf ("Forwards\n");
      rfftwnd_real_to_complex (fplan, 3, data, 1, OD, fft, 1, OF);

      printf ("Writing Power spectrum\n");
      {
        FILE *out = fopen (FRQ, "w");
        int n = NQ * NY;
        fftw_complex *ptr = fft;
        fftw_real l = 0, s = 1e20;
        int c;


        fprintf (out, "P6\n%d %d\n255\n", NQ, NY);


        while (n--)
          {
            fftw_real f = ((ptr->re) * (ptr->re)) + ((ptr->im) * (ptr->im));
            if (f > l)
              l = f;
            ptr++;
          }


        l = log (l);

#define DR 25.0f

#define INNARDS(o) \
        f = (((o)->re) * ((o)->re)) + (((o)->im) * ((o)->im)); \
        f=log(f)-l;\
	f=((f+DR)*230.0f)/DR;\
        c = (int) (f + .5f);\
	if (c<0) c=0;\
	if (c>230) c=230; \
	c+=23;

        n = NQ * NY;
        c = n >> 1;
        ptr = fft;
        n = n - c;
        ptr += c;

        while (n--)
          {
            fftw_real f;
            INNARDS (ptr);
            putc (c, out);
            INNARDS (ptr + OF);
            putc (c, out);
            INNARDS (ptr + OF2);
            putc (c, out);
            ptr++;
          }
        n = NQ * NY;
        ptr = fft;
        n = n >> 1;
        while (n--)
          {
            fftw_real f;
            INNARDS (ptr);
            putc (c, out);
            INNARDS (ptr + OF);
            putc (c, out);
            INNARDS (ptr + OF2);
            putc (c, out);
            ptr++;
          }

#undef INNARDS

        fclose (out);
      }

      kl = fopen (KILL, "r");

      if (!kl)
        {
          printf ("No filter %s\n", KILL);
        }
      else
        {
          printf ("Reading filter\n");
          {
            int n, c;
            fftw_complex *ptr;

            FILE *in = kl;
            if (!in)
              return 0;

            fgetsnc (buf, sizeof (buf), in);
            fgetsnc (buf, sizeof (buf), in);
            fgetsnc (buf, sizeof (buf), in);


            n = NQ * NY;
            c = n >> 1;
            ptr = fft;
            n = n - c;
            ptr += c;

#define INNARDS if (c==0) { ptr->re=0; ptr->im=0; (ptr+OF)->re=0; (ptr+OF)->im=0; (ptr+OF2)->re=0; (ptr+OF2)->im=0; }

            while (n--)
              {
                c = getc (in) ? 1 : 0;
                c = getc (in) ? 1 : c;
                c = getc (in) ? 1 : c;

                INNARDS;
                ptr++;
              }
            n = NQ * NY;
            ptr = fft;
            n = n >> 1;
            while (n--)
              {
                c = getc (in) ? 1 : 0;
                c = getc (in) ? 1 : c;
                c = getc (in) ? 1 : c;

                INNARDS;
                ptr++;
              }

#undef INNARDS

            fclose (in);
          }


          printf ("Reverse\n");

          rfftwnd_complex_to_real (bplan, 3, fft, 1, OF, data, 1, OD);

          printf ("Writing output \n");

          {
            int n = NX * NY;
            fftw_real scale = (fftw_real) (NX * NY);
            fftw_real *ptr = data;
            FILE *out = fopen (OUT, "w");


            ptr = data;

            fprintf (out, "P6\n%d %d\n255\n", NX, NY);


#define INNARDS(o) \
  i=(int) ((*(o))/scale); \
  if (i<0) i=0;\
  if (i>255) i=255;\

            scale = scale / 255.0;

            while (n--)
              {
                int i;

                INNARDS (ptr);
                putc (i, out);
                INNARDS (ptr + OD);
                putc (i, out);
                INNARDS (ptr + OD2);
                putc (i, out);

                ptr++;
              }

            fclose (out);

          }

        }

argv++;
    }


  free (fft);
  free (data);
  rfftwnd_destroy_plan (bplan);
  rfftwnd_destroy_plan (fplan);

  rewind(ws);
  fftw_export_wisdom_to_file(ws);
  fclose(ws);


  return 0;
}
